/* A simplified version of the domctl.h header file
 * with only the information we need.
 * Since our module is not part of the Xen tree
 * and therefore doesn't know about its header files,
 * we had to create our own version. Might as well make that 
 * slimm with only the stuff we need.
 */

#ifndef uint64_aligned_t
#define uint64_aligned_t uint64_t
#endif

#define XEN_DOMCTL_INTERFACE_VERSION 0x00000005

#define XEN_DOMCTL_set_synced        51
#define XEN_DOMCTL_get_synced        52
typedef struct xen_domctl_synced {
  char is_synchronized;
  int sync_done;
  char sync_currently_running;
} xen_domctl_synced_t;
DEFINE_XEN_GUEST_HANDLE(xen_domctl_synced_t);

#define XEN_DOMCTL_getdomaininfo      5
struct xen_domctl_getdomaininfo {
  /* OUT variables. */
  domid_t  domain;              /* Also echoed in domctl.domain */
  /* Domain is scheduled to die. */
#define _XEN_DOMINF_dying     0
#define XEN_DOMINF_dying      (1U<<_XEN_DOMINF_dying)
  /* Domain is an HVM guest (as opposed to a PV guest). */
#define _XEN_DOMINF_hvm_guest 1
#define XEN_DOMINF_hvm_guest  (1U<<_XEN_DOMINF_hvm_guest)
  /* The guest OS has shut down. */
#define _XEN_DOMINF_shutdown  2
#define XEN_DOMINF_shutdown   (1U<<_XEN_DOMINF_shutdown)
  /* Currently paused by control software. */
#define _XEN_DOMINF_paused    3
#define XEN_DOMINF_paused     (1U<<_XEN_DOMINF_paused)
  /* Currently blocked pending an event.     */
#define _XEN_DOMINF_blocked   4
#define XEN_DOMINF_blocked    (1U<<_XEN_DOMINF_blocked)
  /* Domain is currently running.            */
#define _XEN_DOMINF_running   5
#define XEN_DOMINF_running    (1U<<_XEN_DOMINF_running)
  /* CPU to which this domain is bound.      */
#define XEN_DOMINF_cpumask      255
#define XEN_DOMINF_cpushift       8
  /* XEN_DOMINF_shutdown guest-supplied code.  */
#define XEN_DOMINF_shutdownmask 255
#define XEN_DOMINF_shutdownshift 16
  uint32_t flags;              /* XEN_DOMINF_* */
  uint64_aligned_t tot_pages;
  uint64_aligned_t max_pages;
  uint64_aligned_t shared_info_frame; /* GMFN of shared_info struct */
  uint64_aligned_t cpu_time;
  uint32_t nr_online_vcpus;    /* Number of VCPUs currently online. */
  uint32_t max_vcpu_id;        /* Maximum VCPUID in use by this domain. */
  uint32_t ssidref;
  xen_domain_handle_t handle;
};
typedef struct xen_domctl_getdomaininfo xen_domctl_getdomaininfo_t;
DEFINE_XEN_GUEST_HANDLE(xen_domctl_getdomaininfo_t);

#define XEN_DOMCTL_scheduler_op      16
/* Scheduler types. */
#define XEN_SCHEDULER_SEDF     4
#define XEN_SCHEDULER_CREDIT   5
#define XEN_SCHEDULER_SYNCSEDF     6
/* Set or get info? */
#define XEN_DOMCTL_SCHEDOP_putinfo 0
#define XEN_DOMCTL_SCHEDOP_getinfo 1
struct xen_domctl_scheduler_op {
  uint32_t sched_id;  /* XEN_SCHEDULER_* */
  uint32_t cmd;       /* XEN_DOMCTL_SCHEDOP_* */
  union {
    struct xen_domctl_sched_sedf {
      uint64_aligned_t period;
      uint64_aligned_t slice;
      uint64_aligned_t latency;
      uint32_t extratime;
      uint32_t weight;
    } sedf;
    struct xen_domctl_sched_credit {
      uint16_t weight;
      uint16_t cap;
    } credit;
  } u;
};
typedef struct xen_domctl_scheduler_op xen_domctl_scheduler_op_t;
DEFINE_XEN_GUEST_HANDLE(xen_domctl_scheduler_op_t);



struct xen_domctl {
  uint32_t cmd;
  uint32_t interface_version; /* XEN_DOMCTL_INTERFACE_VERSION */
  domid_t  domain;
  union {
    struct xen_domctl_synced            synced;
    struct xen_domctl_getdomaininfo     getdomaininfo;
    struct xen_domctl_scheduler_op      scheduler_op;
    uint8_t                             pad[128];
  } u;
};
typedef struct xen_domctl xen_domctl_t;
DEFINE_XEN_GUEST_HANDLE(xen_domctl_t);
