#ifndef KERNELSYNC_DRIVER_H_
#define KERNELSYNC_DRIVER_H_

#define USE_RWLOCK

#define MAX_CLIENTS 100
#define CLIENT_MAX_DESCRIPTION_LENGTH 100

#define MAX_DOMS 32

#define CLIENT_TYPE_LOCAL_XDOMAIN 0
#define CLIENT_TYPE_REMOTE_XDOMAIN 1
#define CLIENT_TYPE_REMOTE_SIMULATION 2
#define CLIENT_TYPE_UNREGISTERED 99
#define CLIENT_TYPE_TEST 133
#define CLIENT_TYPE_OTHER 254
#define CLIENT_TYPE_UNKNOWN 255

#define UNREGISTER_REASON_REGULAR 0
#define UNREGISTER_REASON_OUT_OF_SYNC 1
#define UNREGISTER_REASON_OTHER 2

#define PACKETTYPE_REGISTER 0
#define PACKETTYPE_UNREGISTER 1
#define PACKETTYPE_RUNPERMISSION 2
#define PACKETTYPE_FINISHED 3

#define PACKETTYPE_DUMMY 142

typedef struct
{
  int id;
  unsigned int seqnr;
  int period;
  rwlock_t lock;
  struct socket *sk;
  struct sockaddr_in servaddr;
  char description[CLIENT_MAX_DESCRIPTION_LENGTH + 1];
} client_context_t;

typedef struct
{
  int is_registered;
  int type;
  uint32_t period;
  char description[CLIENT_MAX_DESCRIPTION_LENGTH + 1];
} client_info_t;

typedef struct
{
  uint32_t period;
  uint32_t barrier_time;
  uint64_t simulation_time;
  rwlock_t lock;
  struct socket *sk;
  struct sockaddr_in cliaddr;
  struct sockaddr_in anyaddr;
  unsigned int seqnr;
  unsigned int client_count;
  unsigned int client_fin_this_period;
  client_info_t client[MAX_CLIENTS];
} server_context_t;

// COM_RegisterClient in synchronization.h
typedef struct
{
  uint16_t id;
  uint8_t type;
  char description[CLIENT_MAX_DESCRIPTION_LENGTH];
} data_reg_t;

// COM_UnregisterClient in synchronization.h
typedef struct
{
  uint16_t id;
  uint8_t reason;
} data_unreg_t;

/* CAREFUL: the two fields "period" and "runtime" of
   data_run_t and data_fin_t, repsectively, 
   MUST STAY ALIGNED so they can be typecast into
   each other, which makes sending a finish reply
   very convenient. */

// COM_RunPermission in synchronization.h
typedef struct
{
  uint32_t period;
  uint32_t runtime;
} data_run_t;

// COM_Finished in synchronization.h
typedef struct
{
  uint32_t period;
  uint32_t runtime; // is this really necessary?
  uint32_t realtime;
  uint16_t id;
} data_fin_t;

// COM_Syncpacket in synchronization.h
typedef struct {
  uint32_t seqnr;
  uint8_t packet_type;
  char data[];
} sync_packet_t;

typedef struct {
  struct socket *sk;
  struct sockaddr_in self_addr;
} client_dummy_context_t;
static int sync_client_init(void);

static int client_thread(void *arg);

int handle_pkt_register(data_reg_t *data);

int handle_pkt_unregister(data_unreg_t *data);

int handle_pkt_finished(data_fin_t *data);

void send_runpermission(void);

int wait_for_runpermission(void);

int send_toserver(sync_packet_t *packet, int len);

static irqreturn_t client_irqhandler(int irq, void *dev_id, struct pt_regs *regs);

static void client_irq_bottomhalf(void *arg);

static void bh_run_synced_domain(void *arg);

static int hypercall_run_synced_domain(int domnr);

static int hypercall_sync_domain(int domnr);

static int hypercall_unsync_domain(int domnr);

static int kernelsync_init_module(void);

static void kernelsync_exit_module(void);

#endif
